@@ -36,15 +36,17 @@ CFAxisDiscrete <- R6::R6Class("CFAxisDiscrete",
   ),
   public = list(
     #' @description Create a new instance of this class.
-    #' @param grp The group that contains the netCDF variable.
+    #'
+    #'   Creating a new discrete axis is more easily done with the
+    #'   [makeDiscreteAxis()] function.
     #' @param nc_var The netCDF variable that describes this instance.
     #' @param nc_dim The netCDF dimension that describes the dimensionality.
     #' @param orientation The orientation (`X`, `Y`, `Z`, or `T`) or `""` if
     #' different or unknown.
     #' @param dim_only Flag if this axis only has a dimension on file but no
     #' NC variable.
-    initialize = function(grp, nc_var, nc_dim, orientation, dim_only = FALSE) {
-      super$initialize(grp, nc_var, nc_dim, orientation)
+    initialize = function(nc_var, nc_dim, orientation, dim_only = FALSE) {
+      super$initialize(nc_var, nc_dim, orientation)
       self$set_attribute("actual_range", "NC_INT", c(1L,self$length))
       private$dim_only <- dim_only
     },
@@ -67,6 +69,23 @@ CFAxisDiscrete <- R6::R6Class("CFAxisDiscrete",
       out
     },
 
+    #' @description Append a vector of values at the end of the current values
+    #'   of the axis. In a discrete axis the values are always a simple sequence
+    #'   so the appended values extend the sequence, rather than using the
+    #'   values from axis `from`.
+    #' @param from An instance of `CFAxisDiscrete` whose length to add to the
+    #'   length of `self`.
+    #' @return A new `CFAxisDiscrete` with the combined length of `self` and the
+    #'   `from` axis.
+    append = function(from) {
+      if (super$can_append(from)) {
+        axis <- makeDiscreteAxis(self$name, makeGroup(), private$length + from$length)
+        axis$attributes <- self$attributes
+        axis
+      } else
+        stop("Axis values cannot be appended.", call. = FALSE)
+    },
+
     #' @description Find indices in the axis domain. Given a vector of numerical
     #'   values `x`, find their indices in the values of the axis. In effect,
     #'   this returns index values into the axis, but outside values will be
@@ -74,13 +93,12 @@ CFAxisDiscrete <- R6::R6Class("CFAxisDiscrete",
     #'
     #' @param x Vector of numeric values to find axis indices for.
     #' @param method Ignored.
+    #' @param rightmost.closed Ignored.
     #'
     #' @return Numeric vector of the same length as `x`. Values of `x` outside
-    #'   of the range of the values in the axis are returned as `0` and
-    #'   `.Machine$integer.max`, respectively.
-    indexOf = function(x, method = "constant") {
-      x[x < 1] <- 0L
-      x[x > self$length] <- .Machine$integer.max
+    #'   of the range of the values in the axis are returned as `NA`.
+    indexOf = function(x, method = "constant", rightmost.closed = TRUE) {
+      x[x < 1 | x > self$length] <- NA
       as.integer(x)
     },
 
@@ -93,36 +111,20 @@ CFAxisDiscrete <- R6::R6Class("CFAxisDiscrete",
     #'   axis.
     #'
     #' @return A `CFAxisDiscrete` instance covering the indicated range of
-    #'   indices. If the `rng` argument includes only a single value, an
-    #'   [CFAxisScalar] instance is returned with the value from this axis. If
-    #'   the value of the argument is `NULL`, return the entire axis (possibly
-    #'   as a scalar axis).
+    #'   indices. If the value of the argument is `NULL`, return the entire
+    #'   axis.
     subset = function(group, rng = NULL) {
       var <- NCVariable$new(-1L, self$name, group, "NC_DOUBLE", 1L, NULL)
 
-      .make_scalar <- function(idx) {
-        scl <- CFAxisScalar$new(group, var, self$orientation, idx)
-        private$subset_coordinates(scl, idx)
-        browser
-        scl
-      }
-
       if (is.null(rng)) {
-        if (self$length > 1L) {
-          ax <- self$clone()
-          ax$group <- group
-          ax
-        } else
-          .make_scalar(1L)
+        ax <- self$clone()
+        ax$group <- group
+        ax
       } else {
-        if (rng[1L] == rng[2L])
-          .make_scalar(rng[1L])
-        else {
-          dim <- NCDimension$new(-1L, self$name, rng[2L] - rng[1L] + 1L, FALSE)
-          discr <- CFAxisDiscrete$new(group, var, dim, self$orientation)
-          private$subset_labels(discr, idx)
-          discr
-        }
+        dim <- NCDimension$new(-1L, self$name, rng[2L] - rng[1L] + 1L, FALSE)
+        discr <- CFAxisDiscrete$new(var, dim, self$orientation)
+        private$subset_labels(discr, idx)
+        discr
       }
     },
 
