@@ -55,7 +55,6 @@ CFVariable <- R6::R6Class("CFVariable",
     auxiliary_interpolation = function(subset, aoi) {
       # This code assumes that the grid orientation of the data variable is the
       # same as that of the longitude-latitude grid
-
       private$llgrid$aoi <- if (is.null(aoi)) {
         ext <- private$llgrid$extent
         Xrng <- if (is.na(subset$X[[1L]])) ext[1:2] else range(subset$X)
@@ -178,13 +177,11 @@ CFVariable <- R6::R6Class("CFVariable",
   ),
   public = list(
     #' @description Create an instance of this class.
-    #'
-    #' @param grp The group that this CF variable lives in.
     #' @param nc_var The netCDF variable that defines this CF variable.
     #' @param axes List of [CFAxis] instances that describe the dimensions.
     #' @return An instance of this class.
-    initialize = function(grp, nc_var, axes) {
-      super$initialize(nc_var, grp, axes, NULL)
+    initialize = function(nc_var, axes) {
+      super$initialize(nc_var, axes, NULL)
       nc_var$CF <- self
 
       # Sanitize attributes for valid range, missing values and packing
@@ -293,114 +290,104 @@ CFVariable <- R6::R6Class("CFVariable",
     },
 
     #' @description This method extracts a subset of values from the array of
-    #' the variable, with the range along each axis to extract expressed in
-    #' values of the domain of each axis.
+    #'   the variable, with the range along each axis to extract expressed in
+    #'   coordinate values of the domain of each axis.
     #'
     #' @details The range of values along each axis to be subset is expressed in
-    #' values of the domain of the axis. Any axes for which no information is
-    #' provided in the `subset` argument are extracted in whole. Values can be
-    #' specified in a variety of ways that are specific to the nature of the
-    #' axis. For numeric axes it should (resolve to) be a vector of real
-    #' values. A range (e.g. `100:200`), a vector (`c(23, 46, 3, 45, 17`),
-    #' a sequence (`seq(from = 78, to = 100, by = 2`), all work. Note, however,
-    #' that only a single range is generated from the vector so these examples
-    #' resolve to `(100, 200)`, `(3, 46)`, and `(78, 100)`, respectively. For
-    #' time axes a vector of character timestamps, `POSIXct` or `Date` values
-    #' must be specified. As with numeric values, only the two extreme values in
-    #' the vector will be used.
+    #'   coordinates of the domain of the axis. Any axes for which no
+    #'   selection is made in the `...` argument are extracted in
+    #'   whole. Coordinates can be specified in a variety of ways that are
+    #'   specific to the nature of the axis. For numeric axes it should (resolve
+    #'   to) be a vector of real values. A range (e.g. `100:200`), a vector
+    #'   (`c(23, 46, 3, 45, 17`), a sequence (`seq(from = 78, to = 100, by =
+    #'   2`), all work. Note, however, that only a single range is generated
+    #'   from the vector so these examples resolve to `(100, 200)`, `(3, 46)`,
+    #'   and `(78, 100)`, respectively. For time axes a vector of character
+    #'   timestamps, `POSIXct` or `Date` values must be specified. As with
+    #'   numeric values, only the two extreme values in the vector will be used.
     #'
-    #' If the range of values for an axis in argument `subset` extend the valid
-    #' range of the axis in `x`, the extracted slab will start at the beginning
-    #' for smaller values and extend to the end for larger values. If the
-    #' values envelope the valid range the entire axis will be extracted in
-    #' the result. If the range of `subset` values for any axis are all either
-    #' smaller or larger than the valid range of the axis in `x` then nothing
-    #' is extracted and `NULL` is returned.
+    #'   If the range of coordinate values for an axis in argument `...`
+    #'   extend the valid range of the axis in `x`, the extracted data will
+    #'   start at the beginning for smaller values and extend to the end for
+    #'   larger values. If the values envelope the valid range the entire axis
+    #'   will be extracted in the result. If the range of coordinate values for
+    #'   any axis are all either smaller or larger than the valid range of the
+    #'   axis then nothing is extracted and `NULL` is returned.
     #'
-    #' The extracted data has the same dimensional structure as the data in the
-    #' variable, with degenerate dimensions dropped. The order of the axes in
-    #' argument `subset` does not reorder the axes in the result; use the
-    #' [CFArray]$array() method for this.
+    #'   The extracted data has the same dimensional structure as the data in
+    #'   the variable, with degenerate dimensions dropped. The order of the axes
+    #'   in argument `...` does not reorder the axes in the result; use the
+    #'   [CFArray]$array() method for this.
     #'
-    #' As an example, to extract values of a variable for Australia for the year
-    #' 2020, where the first axis in `x` is the longitude, the second
-    #' axis is the latitude, both in degrees, and the
-    #' third (and final) axis is time, the values are extracted by
-    #' `x$subset(list(X = c(112, 154), Y = c(-9, -44), T = c("2020-01-01", "2021-01-01")))`.
-    #' You could take the longitude-latitude values from `sf::st_bbox()` or
-    #' `terra::ext()` if you have specific spatial geometries for whom you want to
-    #' extract data. Note that this works equally well for projected coordinate
-    #' reference systems - the key is that the specification in argument `subset`
-    #' uses the same domain of values as the respective axes in `x` use.
+    #'   As an example, to extract values of a variable for Australia for the
+    #'   year 2020, where the first axis in `x` is the longitude, the second
+    #'   axis is the latitude, both in degrees, and the third (and final) axis
+    #'   is time, the values are extracted by
+    #'   `x$subset(X = c(112, 154), Y = c(-9, -44), T = c("2020-01-01", "2021-01-01"))`.
+    #'   You could take the longitude-latitude values from `sf::st_bbox()` or
+    #'   `terra::ext()` if you
+    #'   have specific spatial geometries for whom you want to extract data.
+    #'   Note that this works equally well for projected coordinate reference
+    #'   systems - the key is that the specification in argument `...` uses
+    #'   the same domain of values as the respective axes in `x` use.
     #'
-    #' ## Auxiliary coordinate variables
+    #'   ## Auxiliary coordinate variables
     #'
-    #' A special case exists for variables where the horizontal dimensions (X
-    #' and Y) are not in longitude and latitude values but in some other
-    #' coordinate system. In this case the netCDF resource may have so-called
-    #' *auxiliary coordinate variables* for longitude and latitude that are two
-    #' grids with the same dimension as the horizontal axes of the data variable
-    #' where each pixel gives the corresponding value for the longitude and
-    #' latitude. If the variable has such *auxiliary coordinate variables* then
-    #' they will be used automatically if, and only if, the axes are labeled in
-    #' argument `subset` as `X` and `Y`. The resolution of the grid that is
-    #' produced by this method is automatically calculated. If you want to
-    #' subset those axes then specify values in decimal degrees; if you want to
-    #' extract the full extent, specify `NA` for both `X` and `Y`. **Note** that
-    #' if you want to extract the data in the original grid, you should use the
-    #' horizontal axis names in argument `subset`.
-    #'
-    #' @param subset A list with the range to extract from each axis. The
-    #' list should have elements for the axes to extract a subset from - if an
-    #' axis is not present in the list the entire axis will be extracted
-    #' from the array. List element names should be the axis designator `X`, `Y`,
-    #' `Z` or `T`, or the name of the axis - axes without an axis designator
-    #' and any additional axes beyond the four standard ones can only
-    #' be specified by name. Axis designators and names are case-sensitive and
-    #' can be specified in any order. If values for the range per axis fall
-    #' outside of the extent of the axis, the range is clipped to the extent of
-    #' the axis.
-    #' @param aoi Optional, an area-of-interest instance of class `AOI` created
-    #' with the [aoi()] function to indicate the horizontal area that should be
-    #' extracted. The longitude and latitude coordinates must be included; the X
-    #' and Y resolution will be calculated if not given. When provided, this
-    #' argument will take precedence over the corresponding axis information for
-    #' the X and Y axes in the `subset` argument.
+    #'   A special case exists for variables where the horizontal dimensions (X
+    #'   and Y) are not in longitude and latitude coordinates but in some other
+    #'   coordinate system. In this case the netCDF resource may have so-called
+    #'   *auxiliary coordinate variables* for longitude and latitude that are
+    #'   two grids with the same dimension as the horizontal axes of the data
+    #'   variable where each pixel gives the corresponding value for the
+    #'   longitude and latitude. If the variable has such *auxiliary coordinate
+    #'   variables* then they will be used automatically if, and only if, the
+    #'   axes are labeled in argument `...` as `X` and `Y`. The resolution of
+    #'   the grid that is produced by this method is automatically calculated.
+    #'   If you want to subset those axes then specify values in decimal
+    #'   degrees; if you want to extract the full extent, specify `NA` for both
+    #'   `X` and `Y`. **Note** that if you want to extract the data in the
+    #'   original grid, you should use the horizontal axis names in argument
+    #'   `...`.
+    #' @param ... One or more arguments of the form `axis = range`. The
+    #'   "axis" part should be the name of an axis or its orientation `X`, `Y`,
+    #'   `Z` or `T`. The "range" part is a vector of values representing
+    #'   coordinates along the axis where to extract data. Axis designators and
+    #'   names are case-sensitive and can be specified in any order. If values
+    #'   for the range per axis fall outside of the extent of the axis, the
+    #'   range is clipped to the extent of the axis.
+    #' @param .aoi Optional, an area-of-interest instance of class `AOI` created
+    #'   with the [aoi()] function to indicate the horizontal area that should
+    #'   be extracted. The longitude and latitude coordinates must be included;
+    #'   the X and Y resolution will be calculated if not given. When provided,
+    #'   this argument will take precedence over the corresponding axis
+    #'   information for the X and Y axes in the `subset` argument.
     #' @param rightmost.closed Single logical value to indicate if the upper
-    #' boundary of range in each axis should be included.
-    #' @param ... Ignored. Included to avoid "unused argument" errors on
-    #' argument `rightmost.closed`.
-    #'
+    #'   boundary of range in each axis should be included.
     #' @return A [CFArray] instance, having an array with its axes and
-    #' attributes of the variable, or `NULL` if one or more of the elements in
-    #' the `subset` argument falls entirely outside of the range of the axis.
-    #' Note that degenerate dimensions (having `length(.) == 1`) are dropped
-    #' from the array but the corresponding axis is maintained in the result as
-    #' a scalar axis.
-    subset = function(subset, aoi = NULL, rightmost.closed = FALSE, ...) {
+    #'   attributes of the variable, or `NULL` if one or more of the selectors
+    #'   in the `...` argument fall entirely outside of the range of the axis.
+    #'   Note that degenerate dimensions (having `length(.) == 1`) are dropped
+    #'   from the array but the corresponding axis is maintained in the result
+    #'   as a scalar axis.
+    subset = function(..., .aoi = NULL, rightmost.closed = FALSE) {
       num_axes <- private$num_dim_axes()
       if (!num_axes)
         stop("Cannot subset a scalar variable", call. = FALSE)
-      if (!is.null(aoi) && is.null(aoi$lonMin))
-        stop("Argument `aoi` must have coordinates set", call. = FALSE)
+      if (!is.null(.aoi) && is.null(.aoi$lonMin))
+        stop("Argument `.aoi` must have coordinates set", call. = FALSE)
 
+      # Organize the selectors
+      selectors <- list(...)
+      sel_names <- names(selectors)
       axis_names <- names(self$axes)
-      orientations <- sapply(self$axes, function(a) a$orientation)
-      axis_order <- match(c("X", "Y", "Z", "T"), orientations)
-      names(axis_order) <- c("X", "Y", "Z", "T")
+      axis_order <- private$check_names(sel_names)
 
-      sub_names <- names(subset)
-
-      if ((all(c("X", "Y") %in% sub_names) || !missing(aoi)) && inherits(private$llgrid, "CFAuxiliaryLongLat")) {
-        aux <- private$auxiliary_interpolation(subset, aoi)
-        sub_names <- sub_names[!grepl("X|Y", sub_names)]
+      if ((all(c("X", "Y") %in% sel_names) || !missing(.aoi)) && inherits(private$llgrid, "CFAuxiliaryLongLat")) {
+        aux <- private$auxiliary_interpolation(selectors, .aoi)
+        sel_names <- sel_names[!grepl("X|Y", sel_names)]
       } else aux <- NULL
 
-      bad <- sub_names[!(sub_names %in% c(axis_names, orientations))]
-      if (length(bad))
-        stop("Argument `subset` contains elements not corresponding to an axis:", paste(bad, collapse = ", "), call. = FALSE)
-
-      out_group <- NCGroup$new(-1L, "/", "/", NULL, NULL)
+      out_group <- makeGroup()
       out_group$set_attribute("title", "NC_CHAR", paste("Processing result of variable", self$name))
       out_group$set_attribute("history", "NC_CHAR", paste0(format(Sys.time(), "%FT%T%z"), " R package ncdfCF(", packageVersion("ncdfCF"), ")::CFVariable$subset()"))
 
@@ -411,44 +398,29 @@ CFVariable <- R6::R6Class("CFVariable",
       out_axes_other <- list()
       for (ax in 1:num_axes) {
         axis <- self$axes[[ax]]
+        orient <- axis$orientation
 
-        if (!is.null(aux) && orientations[ax] == "X") {
+        # In every section, set start and count values and create a corresponding axis
+        if (!is.null(aux) && orient == "X") {
           start[ax] <- aux$X[1L]
           count[ax] <- aux$X[2L]
-          var <- NCVariable$new(-1L, private$llgrid$varLong$name, out_group, "NC_DOUBLE", 1L, NULL)
-          var$attributes <- private$llgrid$varLong$attributes
-          out_axis <- if (aux$X[2L] == 1L)
-            CFAxisScalar$new(out_group, var, "X", aux$aoi$dimnames[[2L]])
-          else {
-            dim <- NCDimension$new(-1L, private$llgrid$varLong$name, aux$aoi$dim[2L], FALSE)
-            newax <- CFAxisLongitude$new(out_group, var, dim, aux$aoi$dimnames[[2L]])
-            newax$bounds <- aux$aoi$bounds(out_group)$lon
-            newax
-          }
-        } else if (!is.null(aux) && orientations[ax] == "Y") {
+          out_axis <- makeLongitudeAxis(private$llgrid$varLong$name, out_group, aux$aoi$dimnames[[2L]], aux$aoi$bounds(out_group)$lon$bounds)
+        } else if (!is.null(aux) && orient == "Y") {
           start[ax] <- aux$Y[1L]
           count[ax] <- aux$Y[2L]
-          var <- NCVariable$new(-1L, private$llgrid$varLat$name, out_group, "NC_DOUBLE", 1L, NULL)
-          var$attributes <- private$llgrid$varLat$attributes
-          out_axis <- if (aux$X[2L] == 1L)
-            CFAxisScalar$new(out_group, var, "Y", aux$aoi$dimnames[[1L]])
-          else {
-            dim <- NCDimension$new(-1L, private$llgrid$varLat$name, aux$aoi$dim[1L], FALSE)
-            newax <- CFAxisLatitude$new(out_group, var, dim, aux$aoi$dimnames[[1L]])
-            newax$bounds <- aux$aoi$bounds(out_group)$lat
-            newax
-          }
+          out_axis <- makeLatitudeAxis(private$llgrid$varLat$name, out_group, aux$aoi$dimnames[[1L]], aux$aoi$bounds(out_group)$lat$bounds)
         } else { # No auxiliary coordinates
           rng <- NULL
-          if (!is.null(aoi))
-            rng <- if (orientations[ax] == "X") c(aoi$lonMin, aoi$lonMax)
-          else if (orientations[ax] == "Y") c(aoi$latMin, aoi$latMax)
-          if (is.null(rng)) rng <- subset[[ axis_names[ax] ]]
-          if (is.null(rng)) rng <- subset[[ orientations[ax] ]]
-          if (is.null(rng)) {
+          if (!is.null(.aoi))
+            rng <- if (orient == "X") c(.aoi$lonMin, .aoi$lonMax)
+                   else if (orient == "Y") c(.aoi$latMin, .aoi$latMax)
+                   else NULL
+          if (is.null(rng)) rng <- selectors[[ axis_names[ax] ]]
+          if (is.null(rng)) rng <- selectors[[ orient ]]
+          if (is.null(rng)) { # Axis not specified so take the whole axis
             ZT_dim <- c(ZT_dim, axis$length)
             out_axis <- axis$subset(out_group, NULL)
-          } else {
+          } else { # Subset the axis
             idx <- private$range2index(axis, rng, rightmost.closed)
             if (is.null(idx)) return(NULL)
             start[ax] <- idx[1L]
@@ -459,7 +431,7 @@ CFVariable <- R6::R6Class("CFVariable",
         }
 
         # Collect axes for result
-        if (inherits(out_axis, "CFAxisScalar"))
+        if (out_axis$length == 1L)
           out_axes_other <- append(out_axes_other, out_axis)
         else
           out_axes_dim <- append(out_axes_dim, out_axis)
@@ -474,13 +446,13 @@ CFVariable <- R6::R6Class("CFVariable",
       }
       d <- drop(d)
 
-      # Sanitize the attributes for the result, as required
-      atts <- self$attributes
-      atts <- atts[!(atts$name == "coordinates"), ]     # drop: these have been set in axes
-
-      # Get a proper CRS
-      if (is.null(aux)) crs <- self$crs
-      else {
+      # Sanitize the attributes for the result, as required, and get a CRS
+      if (is.null(aux)) {
+        atts <- self$attributes
+        crs <- self$crs
+      } else {
+        atts <- private$dropCoordinates(self$attributes,
+                                        c(private$llgrid$varLong$name, private$llgrid$varLat$name))
         atts <- atts[!(atts$name == "grid_mapping"), ]  # drop: warped to lat-long
         crs <- NULL
       }
@@ -611,7 +583,7 @@ dimnames.CFVariable <- function(x) {
 #' summer <- pr[, , 173:263]
 #' str(summer)
 "[.CFVariable" <- function(x, i, j, ..., drop = FALSE) {
-  numaxes <- sum(!sapply(x$axes, inherits, "CFAxisScalar"))
+  numaxes <- sum(sapply(x$axes, function(ax) ax$length > 1L))
   t <- vector("list", numaxes)
   names(t) <- dimnames(x)[1:numaxes]
 
@@ -662,7 +634,7 @@ dimnames.CFVariable <- function(x) {
   # Apply dimension data and other attributes
   if (length(x$axes) && length(dim(data)) == length(dnames)) { # dimensions may have been dropped automatically, e.g. NC_CHAR to character string
     dimnames(data) <- dnames
-    ax <- sapply(x$axes, function(x) if (!inherits(x, "CFAxisScalar")) x$orientation)
+    ax <- sapply(x$axes, function(ax) if (ax$length > 1L) x$orientation)
     ax <- ax[lengths(ax) > 0L]
     attr(data, "axis") <- ax
   }
