@@ -40,7 +40,7 @@ NCObject <- R6::R6Class("NCObject",
     print_attributes = function(width = 50L) {
       if (nrow(self$attributes)) {
         cat("\nAttributes:\n")
-        print(.slim.data.frame(self$attributes, width), right = FALSE, row.names = FALSE)
+        print(.slim.data.frame(self$attributes[-1L], width), right = FALSE, row.names = FALSE)
       }
     },
 
@@ -71,7 +71,7 @@ NCObject <- R6::R6Class("NCObject",
     #'   length of 255 characters. UTF-8 characters are not supported in
     #'   attribute names.
     #' @param type The type of the attribute, as a string value of a netCDF data
-    #'   type.
+    #'   type or a user-defined type.
     #' @param value The value of the attribute. This can be of any supported
     #'   type, including a vector or list of values. Matrices, arrays and like
     #'   compound data structures should be stored as a data variable, not as an
@@ -119,7 +119,6 @@ NCObject <- R6::R6Class("NCObject",
         df$value <- value # Preserve lists
         self$attributes <- rbind(self$attributes, df)
       }
-      # FIXME: Flag that attributes have changed so that object is dirty
       invisible(self)
     },
 
@@ -180,7 +179,25 @@ NCObject <- R6::R6Class("NCObject",
           RNetCDF::att.put.nc(nc, nm, attr$name, attr$type, unlist(attr$value))
         }
       invisible(self)
+    },
+
+    #' @description Add names of axes to the "coordinates" attribute, avoiding
+    #' duplicates and retaining previous values.
+    #' @param crds Vector of axis names to add to the attribute.
+    #' @return Self, invisibly.
+    add_coordinates = function(crds) {
+      current <- self$attributes[self$attributes$name == "coordinates", ]
+      if (nrow(current)) {
+        # There is a "coordinates" attribute already so append values
+        new_val <- paste(unique(c(strsplit(current[[1L, "value"]], " ")[[1L]], crds)), collapse = " ")
+        self$attributes[self$attributes$name == "coordinates", ]$value <- new_val
+        self$attributes[self$attributes$name == "coordinates", ]$length <- nchar(new_val)
+      } else
+        # Make a new "coordinates" attribute
+        self$set_attribute("coordinates", "NC_CHAR", crds)
+      invisible(self)
     }
+
   )
 )
 
